import * as ethers from 'ethers';
import { BigNumber, providers, Signer, BigNumberish } from 'ethers';

declare type Maybe<T> = T | undefined;
declare type Value = {
    value: BigNumber;
    formatted: string;
};

declare type Auction = {
    isAuctionLive: boolean;
    highestBidderAddress: Maybe<string>;
    endsAt: Maybe<number>;
    amount: Value;
    id: number;
    link: string;
};
declare type Offer = {
    amount: Value;
    offererAddress: string;
    offerExpiresAt: number;
    link: string;
};
declare type BuyNow = {
    amount: Value;
    link: string;
};
declare type StrategyResult = {
    ownerAddress: string;
    isInEscrow: boolean;
    auction: Maybe<Auction>;
    buyNow: Maybe<BuyNow>;
    offer: Maybe<Offer>;
};

declare type GetMarketInfoOptions = {
    provider: providers.Provider;
    contractAddress: string;
    tokenId: number;
    referrer?: string;
};
declare type GetMarketInfoResult = StrategyResult;
declare function getMarketInfo(options: GetMarketInfoOptions): Promise<GetMarketInfoResult>;

declare type GetBalancesOptions = {
    provider: providers.Provider;
    accountAddress: string;
};
declare type GetBalancesResult = {
    ensName: string;
    ethBalance: Value;
    availableFethBalance: Value;
    lockedFethBalance: Value;
};
declare function getBalances(options: GetBalancesOptions): Promise<GetBalancesResult>;

declare type SendOfferOptions = {
    signer: Signer;
    contractAddress: string;
    tokenId: number;
    amount: string;
    referrer?: string;
    gasMargin?: number;
};
declare function sendOffer(options: SendOfferOptions): Promise<ethers.ContractTransaction>;

declare type AcceptOfferOptions = {
    signer: Signer;
    contractAddress: string;
    tokenId: number;
    amount: string;
    offererAddress: string;
    gasMargin?: number;
};
declare function acceptOffer(options: AcceptOfferOptions): Promise<ethers.ContractTransaction>;

declare type SetBuyNowOptions = {
    signer: Signer;
    contractAddress: string;
    tokenId: number;
    amount: string;
    gasMargin?: number;
};
declare function setBuyNow(options: SetBuyNowOptions): Promise<ethers.ContractTransaction>;

declare type SendBuyNowOptions = {
    signer: Signer;
    contractAddress: string;
    tokenId: number;
    referrer?: string;
    gasMargin?: number;
};
declare function sendBuyNow(options: SendBuyNowOptions): Promise<ethers.ContractTransaction>;

declare type CancelBuyNowOptions = {
    signer: Signer;
    contractAddress: string;
    tokenId: number;
    gasMargin?: number;
};
declare function cancelBuyNow(options: CancelBuyNowOptions): Promise<ethers.ContractTransaction>;

declare type SetReserveAuctionOptions = {
    signer: Signer;
    contractAddress: string;
    tokenId: number;
    amount: string;
    gasMargin?: number;
};
declare function setReserveAuction(options: SetReserveAuctionOptions): Promise<ethers.ContractTransaction>;

declare type SendBidOptions = {
    signer: Signer;
    contractAddress: string;
    tokenId: number;
    amount: string;
    referrer?: string;
    gasMargin?: number;
};
declare function sendBid(options: SendBidOptions): Promise<ethers.ContractTransaction>;

declare type CancelReserveAuctionOptions = {
    signer: Signer;
    contractAddress: string;
    tokenId: number;
    gasMargin?: number;
};
declare function cancelReserveAuction(options: CancelReserveAuctionOptions): Promise<ethers.ContractTransaction>;

declare type FinalizeReserveAuctionOptions = {
    signer: Signer;
    contractAddress: string;
    tokenId: number;
    gasMargin?: number;
};
declare function finalizeReserveAuction(options: FinalizeReserveAuctionOptions): Promise<ethers.ContractTransaction>;

declare type GetApprovalOptions = {
    provider: providers.Provider;
    contractAddress: string;
    ownerAddress: string;
};
declare type GetApprovalResult = boolean;
declare function getApproval(options: GetApprovalOptions): Promise<boolean>;

declare type SetApprovalOptions = {
    signer: Signer;
    contractAddress: string;
    gasMargin?: number;
};
declare function setApproval(options: SetApprovalOptions): Promise<ethers.ContractTransaction>;

declare type GetLastSoldPriceOptions = {
    provider: providers.Provider;
    contractAddress: string;
    tokenId: number;
};
declare type GetLastSoldPriceResult = {
    value: BigNumber;
    formatted: string;
};
declare function getLastSoldPrice(options: GetLastSoldPriceOptions): Promise<GetLastSoldPriceResult>;

declare type CalculatePercentageIncreaseOptions = {
    value: BigNumber;
    margin?: BigNumberish;
};
declare function calculatePercentageIncrease(options: CalculatePercentageIncreaseOptions): BigNumber;

declare class NoSignerError extends Error {
    name: string;
    message: string;
}
declare class NoProviderError extends Error {
    name: string;
    message: string;
}
declare class MarketApprovedError extends Error {
    name: string;
    message: string;
}

export { AcceptOfferOptions, CancelBuyNowOptions, CancelReserveAuctionOptions, FinalizeReserveAuctionOptions, GetApprovalOptions, GetApprovalResult, GetBalancesOptions, GetBalancesResult, GetLastSoldPriceOptions, GetLastSoldPriceResult, GetMarketInfoOptions, GetMarketInfoResult, MarketApprovedError, NoProviderError, NoSignerError, SendBidOptions, SendBuyNowOptions, SendOfferOptions, SetApprovalOptions, SetBuyNowOptions, SetReserveAuctionOptions, StrategyResult, acceptOffer, calculatePercentageIncrease, cancelBuyNow, cancelReserveAuction, finalizeReserveAuction, getApproval, getBalances, getLastSoldPrice, getMarketInfo, sendBid, sendBuyNow, sendOffer, setApproval, setBuyNow, setReserveAuction };
